#include "gideros.h"
#include "lua.h"
#include "lauxlib.h"
#define LUA_LIB
#include <jni.h>
//this is for debugging purpose
//and should be commented out before deployment
//you can log using
//__android_log_print(ANDROID_LOG_DEBUG, "tag", "Output String");
#include <android/log.h>
 
//some configurations of our plugin
static const char* pluginName = "giderosrevmob";
static const char* pluginVersion = "0.9";
static const char* javaClassName = "com/giderosmobile/android/revmob/GiderosRevMob";
 
//Store Java Environment reference
static JNIEnv *ENV;
//Store our main class, what we will use as plugin
static jclass cls;
//store modifyString method ID
static jmethodID jstartSession;
static jmethodID jsetTestingMode;
static jmethodID jshowBanner;
 
//startSession method
static int startSession(lua_State *L)
{
    __android_log_print(ANDROID_LOG_DEBUG, "GiderosRevMobPlugin", "startSession");
     
    //if no Java Env, exit
    if(ENV == NULL) return 0;
     
    //check if there is a class
    if(!cls) return 0;
	
	//if we don't have method yet, try to retrieve it
	if (jstartSession == NULL)
    {
        jstartSession = ENV->GetStaticMethodID(cls, "startSession", "(Ljava/lang/String;Ljava/lang/String;)V");
        if(!jstartSession) return 0;
    }
     
    //next we get 1 argument passed from lua, which we know is a string
    //reference: http://pgl.yoyo.org/luai/i/lua_tostring
    const char *appID = lua_tostring(L, 1);
     
    //we pass this string to Java and execute method
    ENV->CallStaticObjectMethod(cls, jstartSession, ENV->NewStringUTF(appID));
	
	return 1;
}

//startSession method
static int setTestingMode(lua_State *L)
{
    __android_log_print(ANDROID_LOG_DEBUG, "GiderosRevMobPlugin", "setTestingMode");
     
    //if no Java Env, exit
    if(ENV == NULL) return 0;
     
    //check if there is a class
    if(!cls) return 0;
	
	//if we don't have method yet, try to retrieve it
	if (jsetTestingMode == NULL)
    {
        jsetTestingMode = ENV->GetStaticMethodID(cls, "setTestingMode", "(Ljava/lang/String;Ljava/lang/String;)V");
        if(!jsetTestingMode) return 0;
    }
     
    //next we get 1 argument passed from lua, which we know is a string
    //reference: http://pgl.yoyo.org/luai/i/lua_tostring
    const char *testingMode = lua_tostring(L, 1);
     
    //we pass this string to Java and execute method
    ENV->CallStaticObjectMethod(cls, jsetTestingMode, ENV->NewStringUTF(testingMode));
	
	return 1;
}

//showBanner method
static int showBanner(lua_State *L)
{
    __android_log_print(ANDROID_LOG_DEBUG, "GiderosRevMobPlugin", "showBanner");
     
    //if no Java Env, exit
    if(ENV == NULL) return 0;
     
    //check if there is a class
    if(!cls) return 0;
	
	//if we don't have method yet, try to retrieve it
	if (jshowBanner == NULL)
    {
        jshowBanner = ENV->GetStaticMethodID(cls, "showBanner", "()V");
        if(!jshowBanner) return 0;
    }
         
    //we pass this string to Java and execute method
    ENV->CallStaticVoidMethod(cls, jshowBanner);
	
	return 1;
}
 
//here we register all functions we could call from lua
//lua function name as key and C function as value
static const struct luaL_Reg funcs[] = {
  { "startSession",  startSession },
  { "setTestingMode",  setTestingMode },
  { "showBanner",  showBanner },
  { NULL, NULL }//don't forget nulls at the end
};
 
//here we register all the C functions for lua
//so lua engine would know they exists
LUALIB_API int luaopen_plugin(lua_State *L)
{
  luaL_register(L, pluginName, funcs);
  return 1;
}
 
//here we do all our stuff needs to be done on initialization
static void g_initializePlugin(lua_State *L)
{
    //get java environment reference
    ENV = g_getJNIEnv();
     
	jclass class_local = ENV->FindClass(javaClassName);
	if(class_local)
	{
		cls = static_cast<jclass>(ENV->NewGlobalRef(class_local)); // <-- Create the global reference
		ENV->DeleteLocalRef(class_local); // <-- Delete the local reference
	}
	 
    //get global package object
    lua_getglobal(L, "package");
    lua_getfield(L, -1, "preload");
     
    //put our plugin name inside with a callback to
    //registering C functions
    lua_pushcfunction(L, luaopen_plugin);
    lua_setfield(L, -2, pluginName);
 
    lua_pop(L, 2);
}
 
//and here we free everything we need to free
static void g_deinitializePlugin(lua_State *L)
{
 
}
 
//register our plugin with Gideros lib
REGISTER_PLUGIN(pluginName, pluginVersion)
//</android></jni.h>